# Reference
<!-- DO NOT EDIT: This document was generated by Puppet Strings -->

## Table of Contents

**Classes**

* [`sap`](#sap): Installs prerequisites for an SAP environment on RedHat derivatives
* [`sap::cluster`](#sapcluster): Configures a node with the appropriate SAP cluster resources
* [`sap::cluster::prepare`](#sapclusterprepare): Ensure local SAP configuration is ready for a clustered environment
* [`sap::config`](#sapconfig): Private class used to control configuration file deployment.
* [`sap::config::common`](#sapconfigcommon): Trick SAP into thinking this host is running a supported RHEL
* [`sap::config::limits`](#sapconfiglimits): Generates an etc limits file for each relevant enabled component.
* [`sap::config::mount_points`](#sapconfigmount_points): Creates and manages mountpoints relevant to each instance type.
* [`sap::config::router`](#sapconfigrouter): Class: sap::config::router  This class contain the configuration for SAP Router  Parameters:   This module has no parameters  Actions:      T
* [`sap::config::sysctl`](#sapconfigsysctl): Configures sysctl parameters for the selected components.
* [`sap::config::tmpfs`](#sapconfigtmpfs): Ensures that the tmpfs partition is configured per SAP recommendations Add swap space check per
* [`sap::install`](#sapinstall): Installs the packages associated with the selected sets
* [`sap::install::mount_dependencies`](#sapinstallmount_dependencies): Installs dependent modules needed for mount points
* [`sap::management`](#sapmanagement): Manages configuration post install of SAP.
* [`sap::management::ads_fonts`](#sapmanagementads_fonts): Deploys font and XDC files to ADS systems
* [`sap::management::db2_queries`](#sapmanagementdb2_queries): Implementation detail - creates sql query scheduling infrastructure
* [`sap::management::dotfiles`](#sapmanagementdotfiles): Manages the dotfiles belonging to SAP users.
* [`sap::management::etc_services`](#sapmanagementetc_services): Creates all relevant etc services entries for this system
* [`sap::management::firewall`](#sapmanagementfirewall): Creates all relevant firewall rules for this system
* [`sap::management::permissions`](#sapmanagementpermissions): Implementation detail - corrects permissions as needed
* [`sap::management::profiles`](#sapmanagementprofiles): Creates the profiles files in /sapmnt/SID/profile for each SID
* [`sap::params`](#sapparams): This module contains the parameters for SAP Netweaver
* [`sap::service`](#sapservice): Configures OS level services required for SAP Netweaver instances
* [`sap::service::cloudconnector`](#sapservicecloudconnector): SAP Cloud Connector service configuration
* [`sap::service::router`](#sapservicerouter): Configures the SAP Router service

**Defined types**

* [`sap::cluster::as`](#sapclusteras): 
* [`sap::cluster::as_inst`](#sapclusteras_inst): Creates Corosync/Pacemaker cluster rules for a single AS instance.
* [`sap::cluster::db2`](#sapclusterdb2): Creates Corosync/Pacemaker cluster rules for a SAP DB2 HA/DR instance.
* [`sap::cluster::fence_lpar`](#sapclusterfence_lpar): Creates Corosync/Pacemaker rules for a fence_lpar STONITH device
* [`sap::cluster::filesystem`](#sapclusterfilesystem): Creates Corosync/Pacemaker cluster rules for a filesystem resource.
* [`sap::cluster::lvm`](#sapclusterlvm): Creates Corosync/Pacemaker cluster rules for an LVM resource.
* [`sap::cluster::scsers`](#sapclusterscsers): Creates Corosync/Pacemaker cluster rules for an (A)SCS/ERS instance.
* [`sap::config::mount_point`](#sapconfigmount_point): Declares a single directory or mountpoint.
* [`sap::management::db2_query_schedule`](#sapmanagementdb2_query_schedule): Implementation detail - a query file & associate cron schedule

**Functions**

* [`multiregsubst`](#multiregsubst): Performs a series of substitutions on the same target string
* [`sap::cluster::filter_nodes`](#sapclusterfilter_nodes): Simple function to retrieve only the nodes with a desired role
* [`sap::sysctl_calculated_values`](#sapsysctl_calculated_values): Recursive function used to compute the value for a sysctl var.
* [`sap::sysctl_line`](#sapsysctl_line): This function converts a sysctl entry hash into a string line.

## Classes

### sap

This module manages SAP prerequisites for several types of SAP installations
based on corresponding SAP OSS notes

#### Examples

##### Application server containing an ADS instance and a Netweaver instance

```puppet
class { 'sap':
  system_ids       => {
    'PR0'          => {
      'components'       => [
         'base',
         'base-extended',
         'ads',
      ],
      'backend_database' => [ 'db2' ],
    },
  },
}
```

#### Parameters

The following parameters are available in the `sap` class.

##### `system_ids`

Data type: `Hash[Sap::SID, Sap::SIDConfigEntry]`

An hash of SAP system IDs (SIDs) which will be present on the target
host along with the following possible attributes for each SID:
* `components` - a list of associated sap applications to enable for this
                 SID (mandatory)
* `backend_databases` - types of databases to which this system connects
                        (mandatory for SIDs with 'base' enabled)
* `manage_profiles` - Flag indicating whether the instance profiles for this
                      should be managed by the module. If specified
                      `instances`, `databases`, and `primary_backend_db`
                      become mandatory parameters!
* `exclude_common_mount_points` - flag indicating if this SID should skip
                                  SID specific entries for the 'common'
                                  mountpoint. Some system types do not have
                                  /sapmnt directories (e.g. a DAA instance
                                  or liveCache)
* `pattern_map` - A hash mapping pattern lookup keys to regular expressions
                  which are unique to this SID. Note that if one is added it
                  must have a corresponding value added in `pattern_values`
* `pattern_values` - Mapping from a pattern lookup key to the value which
                     should be provided wherever the corresponding pattern
                     is found.
* `instances` - Hash detailing the instances which are part of this SID.
                Entries are keyed by `instance class` and contain the
                corresponding Sap::SidInstance detail. See [Data Types in
                the readme](./README.md#data-types) for detail.
* `databases` - Hash detailing the database parameters for this SID.
                Entries in this hash are keyed by `instance class`. For
                detail, see [Data Types in the readme](./README.md#data-types)
* `primary_backend_db` - Sap::InstClassDb of the primary backend database
                         for this node
* `default_pfl_entries` - Hash of profile keys and the desired value which
                          will be appended to the end of the DEFAULT.PFL
                          file for this instance. Currently not validated.
* `sec_info_rules` - Array of Sap::SecInfo entries which will be inserted
                     before the final rules in the secinfo file.
* `reg_info_rules` - Array of Sap::RegInfo entries which will be inserted
                     before the final rules in the reginfo file.
* `db2_query_schedules` - Array of Sap::Db2QuerySchedule entries describing
                          a query script file, associated parameters, and
                          the desired cron schedule entry for each. These
                          queries are then configured to execute as the
                          DB2 instance owner on the host system.

Note that each entry must be exactly 3 characters in length and contain
exclusively uppercase characters. Also, if `base` is included in the
components list for an SID at least one backend_database entry must be
provided.

##### `pattern_map`

Data type: `Hash[String, String]`

Hash mapping a given global key such as 'sid_upper' to the corresponding
regexp used to substitute this value in standard strings. See the
mount_point defined type for detail. Note that the values in this hash
will be converted to regular expressions via `Regexp($value)`

##### `create_mount_points`

Data type: `Boolean`

Indicates whether the standard mount points should be created for the
specified instance.

Default value: `false`

##### `mount_points`

Data type: `Hash[Sap::SapComponent, Sap::MountPoints]`

Defines the mount points and supporting directories which should be created
for each component type. Note that this structure is a deep hash with a `--`
knockout value.

Default value: {}

##### `router_oss_realm`

Data type: `Optional[String]`

Specify OSS realm for SAP router connection. For example,
`'p:CN=hostname.domain.tld, OU=0123456789, OU=SAProuter, O=SAP, C=DE'`

Default value: `undef`

##### `manage_mount_dependencies`

Data type: `Boolean`

When enabled this module will install and configure the puppet-nfs module
detailed here: https://forge.puppet.com/derdanne/nfs
Note that currently only NFSv4 is supported for clients.

Default value: `false`

##### `router_rules`

Data type: `Optional[Array[String]]`

Specify array of rules for the SAP router

Default value: `undef`

##### `distro_text`

Data type: `Optional[String]`

Modify text in /etc/redhat-release

Default value: `undef`

##### `component_packages`

Data type: `Hash[Sap::SapComponent, Array[String]]`

Hash mapping each component type to an array containing the names of the
requried packages on this operating system.

Default value: {}

##### `backenddb_packages`

Data type: `Hash[Sap::BackendDatabase, Array[String]]`

Hash mapping each backend database type to an array containing the names
of the requried packages on this operating system.

Default value: {}

##### `message_servers`

Data type: `Hash[Sap::SID, Sap::MessageServer]`

Hash detailing the message server connection definitions for other systems
this particular instance will need to communicate with. For entry structure
detail See [Data Types in the readme](./README.md#data-types).

Default value: {}

##### `bash_source_profile`

Data type: `Boolean`

When enabled the `.profile` generated by the SAP installer will be sourced
by the .bash_profile of each sidadm user if that file exists.

Default value: `false`

##### `manage_firewall`

Data type: `Boolean`

When enabled all relevant firewall rules for the local system will be
created and managed via firewalld.

Default value: `false`

##### `deployed_fonts`

Data type: `Sap::SourcedFile`

Hash with a list of fonts to deploy and their source locations. Each file in
this structure will be installed in
`/usr/sap/${SAPSYSTEMNAME}/SYS/global/AdobeDocumentServices/FontManagerService/fonts/customer/`
for each local SID with the ADS component enabled.

Default value: {}

##### `deployed_xdc`

Data type: `Sap::SourcedFile`

Hash with a list of xdc files to deploy and their source locations. Each file in
this structure will be installed in
`/usr/sap/${sid}/SYS/global/AdobeDocumentServices/lib/XDC/Customer/`
for each local SID with the ADS component enabled.

Default value: {}

### sap::cluster

Cluster class which configures a PCS/Corosync based ha-environment. Note that
it is assumed that the puppet-corosync class is defined and the cluster is
configured before this class is configured.

Additionally, the cluster must be an Asymmetrical "Opt-In" cluster. This
configuration operates under that assumption and sets cluster level parameters
appropriately.

#### Parameters

The following parameters are available in the `sap::cluster` class.

##### `cib_name`

Data type: `String`

Name of the temporary Cluster Information Base to use during the building
and commiting of the rule changes incorporated in this module.

Default value: `undef`

##### `resource_stickiness`

Data type: `Integer`

Global configuration setting controlling how much a given resource wants to
'stay' where it is as opposed to moving to a 'home' node.

Default value: 500

##### `enable_fencing`

Data type: `Boolean`

Controls whether the cluster assumes Stonith should be enabled or not.

Default value: `false`

##### `sid_rules`

Data type: `Sap::SIDClusterData`

A hash mapping each cluster relevant SID to two mandatory components:
*node_roles*: a complete list of PCS nodes mapped to their roles
*components*: cluster elements for this instance with their data

Default value: {}

##### `fence_agents`

Data type: `Optional[Array[Sap::ClusterFence]]`

An array of Sap::ClusterFence definitions which provide fencing coverage for
all nodes defined by this cluster. Note that this parameter becomes
mandatory when enable_fencing is true. Additionally, every node referenced
within the sid_rules structure must have exactly one corresponding entry in
the pcmk_host_map structure of a single agent defined here.

Default value: []

### sap::cluster::prepare

Simple class which prepares the config files on this machine for a
high-availability configuration of SAP. There are currently two components to
this process: 1) the profile files in /sapmnt/<SID> corresponding to this
machine are updated replacing entries for Restart_Program with Start_Program.
2) All `.sapenv_${HOSTNAME}.(sh|csh)` and other similar host-specific profile
files are removed so the system will only use the `.sapenv.(sh|csh)` variants.

Configuration is performed entirely based on the values of local 'sap' facts.

#### Parameters

The following parameters are available in the `sap::cluster::prepare` class.

##### `cluster_sid_list`

Data type: `Array[Sap::Sid]`

Cluster configuration steps will be taken for each local instance matching
the provided SID list. If multiple SAP instances are located on a given node
only those which match this list will be updated.

##### `packages`

Data type: `Optional[Array[String]]`

Ensures that the resource agents needed for SAP configuration are installed
on this system. Note that defaults are provided only for RHEL 7 derivatives.

Default value: []

### sap::config

Orchestrates the creation of relevant configuration files needed by SAP

### sap::config::common

Overwrites the distro text file to trick SAP into thinking this is RHEL.

### sap::config::limits

This class uses data from hiera to generate an etc limits file for each
enabled component. Note that some components do not specify limits while
others may have SID specific data which will result in a more complicated
limit output

### sap::config::mount_points

Creates and manages mountpoints relevant to each instance type.

### sap::config::router

Class: sap::config::router

This class contain the configuration for SAP Router

Parameters:   This module has no parameters

Actions:      This module has no actions

Requires:     This module has no requirements

Sample Usage: include sap_router::config

### sap::config::sysctl

This class configures the sysctl parameters based on the values
provided in sap::param::config_sysctl for each enabled component

### sap::config::tmpfs

Ensures that the tmpfs partition is configured per SAP recommendations
Add swap space check per

### sap::install

Packages are installed based on the selections provided to the main sap class
using sane defaults configured in hiera.

### sap::install::mount_dependencies

When enabled this ensures that all necessary dependencies for mount point
management are installed

### sap::management

This package manages configuration which is generated during the SAP install.
Note that it must not be applied until after the base 'sap' class.
Additionally, much of the functionality of this subclass is ignored if the
base SAP facts do not seem to indicate that the class has been installed and
configured.

### sap::management::ads_fonts

Deploys font and XDC files to ADS systems

#### Parameters

The following parameters are available in the `sap::management::ads_fonts` class.

##### `system_ids`

Data type: `Hash[Sap::SID, Sap::SIDConfigEntry]`

Sourced directly from the parent system_ids parameter on the SAP class. See
that class for further detail.

Default value: $sap::system_ids

##### `deployed_fonts`

Data type: `Sap::SourcedFile`

Hash with a list of fonts to deploy and their source locations. Each file in
this structure will be installed in
`/usr/sap/${SAPSYSTEMNAME}/SYS/global/AdobeDocumentServices/FontManagerService/fonts/customer/`
for each local SID with the ADS component enabled.

Default value: $sap::deployed_fonts

##### `deployed_xdc`

Data type: `Sap::SourcedFile`

Hash with a list of xdc files to deploy and their source locations. Each file in
this structure will be installed in
`/usr/sap/${sid}/SYS/global/AdobeDocumentServices/lib/XDC/Customer/`
for each local SID with the ADS component enabled.

Default value: $sap::deployed_xdc

### sap::management::db2_queries

Implementation detail - creates sql query scheduling infrastructure

#### Parameters

The following parameters are available in the `sap::management::db2_queries` class.

##### `system_ids`

Data type: `Hash[Sap::SID, Sap::SIDConfigEntry]`

Replicated from sap::system_ids primarily to make the test code easier to
write. See the parent for further detail.

Default value: $sap::system_ids

### sap::management::dotfiles

This is an internal class and should not be called directly.
If $bash_source_profile is set to true for the instance as a whole this class
will be enabled and will cause the .bash_profile of the sid_adm and db2 users
to source their '.profile'.

### sap::management::etc_services

This class is an implementation detail and should not be called directly.

#### Parameters

The following parameters are available in the `sap::management::etc_services` class.

##### `message_servers`

Data type: `Hash[Sap::SID, Sap::MessageServer]`

Sourced directly from the parent message_servers parameter on the SAP class.
See that for further detail.

##### `system_ids`

Data type: `Hash[Sap::SID, Sap::SIDConfigEntry]`

Sourced directly from the parent system_ids parameter on the SAP class. See
that for further detail.

### sap::management::firewall

This class is an implementation detail and should not be called directly.

#### Parameters

The following parameters are available in the `sap::management::firewall` class.

##### `system_ids`

Data type: `Hash[Sap::SID, Sap::SIDConfigEntry]`

Sourced directly from the parent system_ids parameter on the SAP class. See
that class for further detail.

##### `service_map`

Data type: `Hash[Sap::InstClass, Array[Sap::FirewallServices]]`

Mapping from instance class types to the corresponding SAP firewall service
names. If a given instance class is detected on the server it will present
configure the corresponding firewall services.

##### `services`

Data type: `Hash[Sap::FirewallServices, Sap::FirewallRule]`

Hash defining each firewall service entry

### sap::management::permissions

Implementation detail - corrects permissions as needed

#### Parameters

The following parameters are available in the `sap::management::permissions` class.

##### `system_ids`

Data type: `Hash[Sap::SID, Sap::SIDConfigEntry]`

Replicated from sap::system_ids primarily to make the test code easier to
write. See the parent for further detail.

Default value: $sap::system_ids

### sap::management::profiles

This class is an implementation detail and should not be called directly.
Currently only the DEFAULT.PFL file is managed by this class.

#### Parameters

The following parameters are available in the `sap::management::profiles` class.

##### `system_ids`

Data type: `Hash[Sap::SID, Sap::SIDConfigEntry]`

Sourced directly from the parent system_ids parameter on the SAP class. See
that class for further detail.

### sap::params

General parameter definitions for the SAP Netweaver environment are defined
within this module. Note that OS and Architecture specific variants are
actually pulled from defaults in Hiera.

#### Parameters

The following parameters are available in the `sap::params` class.

##### `config_redhat_release_conf`

Data type: `String`

This should point to the path of the desired redhat release config file.

Default value: `undef`

##### `service_uuidd`

Data type: `String`

Name of the uuidd service. This is used to enable the daemon after the
installation completes.

Default value: `undef`

##### `service_scc`

Data type: `String`

SAP Cloud Connector service name used for enabling the service.

Default value: `undef`

##### `service_saprouter`

Data type: `String`

Name of the service corresponding to the SAP router process.

Default value: `undef`

##### `config_saproutetab`

Data type: `String`

File target for the sap router route-table should it be configure.

Default value: `undef`

##### `config_saproutetab_template`

Data type: `String`

Path within the module to the appropriate erb template to apply for the
saproutetab.

Default value: `undef`

##### `config_saprouter_sysconfig`

Data type: `String`

Target /etc/sysconfig file for the saprouter service

Default value: `undef`

##### `config_saprouter_sysconfig_template`

Data type: `String`

Embedded ruby template to be applied to the sysconfig file

Default value: `undef`

##### `config_saprouter_sysconfig_template`

Embedded ruby template to be applied to the sysconfig file

Default value: `undef`

##### `config_sysctl`

Data type: `Hash[
    String,
    Hash[
      String,
      Variant[
        String,
        Integer,
        Array[String],
        Hash[
          String,
          Variant[
            Hash[
              String,
              Variant[
                String,
                Integer,
                Array[
                  Variant[
                    String,
                    Hash[
                      String,
                      Variant[
                        String,
                        Integer,
                      ]
                    ],
                  ]
                ]
              ]
            ]
          ]
        ]
      ]
    ]
  ]`

Similar to config_limits this provides a configurable set of directives to
include in /etc/sysctl.d/ on a per component basis.

Default value: {}

##### `config_limits`

Data type: `Hash[
    String,
    Hash[
      String,
      Variant[
        String,
        Boolean,
        Array[String],
        Hash[
          String,
          Hash[
            String,
            Hash[
              String,
              Variant[
                String,
                Integer,
              ]
            ]
          ]
        ]
      ]
    ]
  ]`

A deeply nested hash containing the target /etc/security/limits.d/
entries for a given system. This is currently defined for baseline sap
instances and db2. The hash has the following structure

```puppet
'component-name' => {
  path           => '/etc/security/limits.d', # this should be left alone - limits directory
  sequence       => '00', # string ID prepended to the limits file. e.g.
                          # /etc/security/limits.d/00-sap-base.conf
  per_sid        => true, # Indicates whether the domain has a _sid_
                          # substring that will need to be substituted with the actual system ID.
  header_comment => [ # An array of comment lines which will appear at the
                      #start of the file
    'Comment line 1',
    'Comment line 2',
  ],
  entries => { # Hash containing the actual
  }
```

Default value: {}

##### `config_default_mount_options`

Data type: `Hash[
    String,
    Hash[
      String,
      String,
    ]
  ]`

Default options to provide to provide for the mount operation in the case
that they are not overridden locally. See
https://puppet.com/docs/puppet/latest/types/mount.html#mount-attribute-options
for details. Note that this is a hash of hashes with an entry for each
supported resource type.

Default value: {}

### sap::service

Configures OS level services required for SAP Netweaver instances

### sap::service::cloudconnector

SAP Cloud Connector service configuration

### sap::service::router

Configures the SAP Router service

## Defined types

### sap::cluster::as

The sap::cluster::as class.

#### Examples

##### Asymmetric 4-node cluster defining the AS resource for each app node

```puppet
sap::cluster::as { 'PR0_As':
  sid          => 'PR0',
  cib_name     => 'sap',
  nodes    => {
    'pr0-app00.example.org' => ['app-standard'],
    'pr0-app01.example.org' => ['app-standard'],
    'pr0-dba.example.org'   => ['db-standard'],
    'pr0-dbb.example.org'   => ['db-standard'],
  },
  as_instances => [
    {
      'type'   => 'DVEBMGS',
      'number' => '00',
      'host'   => 'pr0-app00',
    },
    {
      'type'   => 'D',
      'number' => '00',
      'host'   => 'pr0-app01',
    },
  ],
}
```

#### Parameters

The following parameters are available in the `sap::cluster::as` defined type.

##### `cib_name`

Data type: `String`

Name of the temporary Cluster Information Base to use during the building
and commiting of the rule changes incorporated in this module.

##### `nodes`

Data type: `Sap::ClusterNodeRoles`

Mapping containing each a list of relevant roles for each node in the
cluster.

##### `as_instances`

Data type: `Array[Sap::InstData]`

Each entry in this array corresponds to a unique standalone instance which
is linked to a home host. Entries must have the following three values:
* `type' - 'Name' of the instance type. For example 'D', 'DVEBMGS', or 'J'
* `number' - two digit code assigned to the instance.
* `host' - short hostname used to install the instance.

##### `db_mode`

Data type: `Sap::ClusterDbMode`

Specifies the type of clustering used for the underlying database. This
information is use to construct the start-after relationship on the core
database PCS resource.

##### `sid`

Data type: `Sap::SID`



### sap::cluster::as_inst

This defined type should be used by the sap::cluster class and not called
directly externally.

#### Examples

##### Definition for an instance on pr0-app01

```puppet
sap::cluster::as_inst { 'as_inst_PR0_0':
  sid            => 'PR0',
  cib_name       => 'sap',
  scs_primitive  => 'ms_prm_PR0_SCS',
  db_mode        => 'log_shipping',
  profile_dir    => '/sapmnt/PR0/profile',
  relevant_nodes => ['pr0-app00.example.org', 'pr0-app01.example.org'],
  type           => 'DVEBMGS',
  number         => '00',
  host           => 'pr0-app01',
}
```

#### Parameters

The following parameters are available in the `sap::cluster::as_inst` defined type.

##### `sid`

Data type: `Sap::SID`

System ID this instance is associated with

##### `cib_name`

Data type: `String`

Name of the temporary Cluster Information Base to use during the building
and commiting of the rule changes incorporated in this module.

##### `db_mode`

Data type: `Sap::ClusterDbMode`

Pass-through parameter from the parent sap::cluster::as type. See that
description for details.

##### `profile_dir`

Data type: `Stdlib::AbsolutePath`

Location storing the profile files for each instance. Typically this path is
defined as `/sapmnt/${SID}/profile`. The full start profile is constructed
using the instance specific data provided in other parameters.

##### `relevant_nodes`

Data type: `Array[Stdlib::Host]`

A filtered list of nodes which are relevant to the AS instance described by
this instance. One entry in this must match the `host` parameter defined
below or the declaration will fail.

##### `type`

Data type: `Sap::InstType`

'Name' of the instance type. For example 'D', 'DVEBMGS', or 'J'.

##### `number`

Data type: `Sap::InstNumber`

Two digit code assigned to the instance.

##### `host`

Data type: `Stdlib::Host`

Short hostname used when the instance was installed.

##### `legacy_start_profile`

Data type: `Boolean`

Indicates whether this instance uses the separate START_* profiles as was
common on older releases of SAP Netweaver prior to 7.10. More recent builds
include the START details within the standard profile. For further detail
see https://launchpad.support.sap.com/#/notes/1751381.

Default value: `false`

### sap::cluster::db2

This defined type should be used by the sap::cluster class and not called
directly externally.

#### Examples

##### Asymmetric 4-node cluster with DB2 HADR on two of the nodes

```puppet
sap::cluster::scsers { 'PR0_Db2':
  sid          => 'PR0',
  cib_name     => 'sap',
  nodes        => {
    'pr0-app00.example.org' => ['app-standard'],
    'pr0-app01.example.org' => ['app-standard'],
    'pr0-dba.example.org'   => ['db-standard'],
    'pr0-dbb.example.org'   => ['db-standard'],
  },
  db_ip        => '10.0.128.15',
  mode         => 'log_shipping',
}
```

#### Parameters

The following parameters are available in the `sap::cluster::db2` defined type.

##### `sid`

Data type: `Sap::SID`

System ID this instance is associated with

##### `cib_name`

Data type: `String`

Name of the temporary Cluster Information Base to use during the building
and commiting of the rule changes incorporated in this module.

##### `nodes`

Data type: `Sap::ClusterNodeRoles`

Mapping containing each a list of relevant roles for each node in the
cluster.

##### `db_ip`

Data type: `Stdlib::IP::Address::Nosubnet`

Service IP address which moves between nodes and represents the 'master'
of the cluster. Clients will connect to this instead of
the local IPs of the machines.

##### `mode`

Data type: `Sap::ClusterDbMode`

Indicates the type of HA cluster. There are currently two supported modes
* `log_shipping` - Shared nothing approach which utilizes log shipping
    to keep one or more replicas synchronized with the current primary.
* `shared_storage` - Approach which moves disks containing the
    database instance between one or more nodes.

##### `volumes`

Data type: `Array[Sap::ClusterLvm]`

Array containing a series of hashes which describe the LVM cluster resources
needed for the primary node of this db2 cluster to function.

##### `filesystems`

Data type: `Array[Sap::ClusterFilesystem]`

Array containing a series of hashes which describe the Filesystem cluster
resources needed on the primary node. The parent LVM volume for each file
system must be included in the `volume` resource.

##### `hadr_peer_window`

Data type: `Optional[Integer]`

Maximum number of seconds the standby nodes can be in an HADR peer topology.
This is used to construct 'sane' monitoring intervals for the log-shipping
based DB2 model.

Default value: 300

### sap::cluster::fence_lpar

This defined type should be used by the sap::cluster class and not called
directly externally.

#### Examples

##### Asymmetric 4-node cluster defining the AS resource for each app node

```puppet
sap::cluster::as { 'fence_lpar_0':
  cib_name                  => 'sap',
  ipaddr                    => 'hmc0.example.org',
  login                     => 'service-fence',
  managed                   => 'host0',
  identity_file             => '/root/.ssh/hmc_secret.rsa',
  pcmk_delay_max            => 15,
  pcmk_host_map             => {
    'pr0-app00.example.org' => 'pr0-app00',
    'pr0-app01.example.org' => 'pr0-app01',
    'pr0-dba.example.org'   => 'pr0-dba',
    'pr0-dbb.example.org'   => 'pr0-dbb',
  },
}
```

#### Parameters

The following parameters are available in the `sap::cluster::fence_lpar` defined type.

##### `cib_name`

Data type: `String`

Name of the temporary Cluster Information Base to use during the building
and commiting of the rule changes incorporated in this module.

##### `ipaddr`

Data type: `Stdlib::Host`

IP address or hostname of the fencing HMC

##### `login`

Data type: `String`

Login username for the HMC

##### `managed`

Data type: `Stdlib::Host`

Name of the CEC or managed system which hosts the target LPAR(s)

##### `identity_file`

Data type: `Stdlib::AbsolutePath`

Private key file used to connect to the HMC

##### `pcmk_delay_max`

Data type: `Integer`

a random maximum delay for stonith actions to prevent double fencing

##### `as_instances`

Each entry in this array corresponds to a unique standalone instance which
is linked to a home host. Entries must have the following three values:
* `type' - 'Name' of the instance type. For example 'D', 'DVEBMGS', or 'J'
* `number' - two digit code assigned to the instance.
* `host' - short hostname used to install the instance.

##### `pcmk_host_map`

Data type: `Sap::ClusterFenceHostMap`

One to one mapping from trusted node host names to the LPAR names in the
HMC. An individual stonith resource will be created for each entry in this
list and will be allowed to run on any node other than the node it is
responsible for fencing.

##### `node_list`

Data type: `Array[Stdlib::Host]`

Full list of nodes in the cluster. This is used to co-locate the resource on
any node other than the home node.

### sap::cluster::filesystem

This defined type should be used by the sap::cluster class and not called
directly externally.

#### Examples

##### Definition for a shared commvault volume

```puppet
sap::cluster::as_inst { 'db2pr0_lvm_0':
  sid            => 'PR0',
  cib_name       => 'sap',
  volume_group   => 'db2cv',
  logical_volume => 'db2cvarch,
  fstype         => 'xfs',
  directory      => '/db2/PR0/commvault/archive',
}
```

#### Parameters

The following parameters are available in the `sap::cluster::filesystem` defined type.

##### `sid`

Data type: `Sap::SID`

System ID this instance is associated with

##### `cib_name`

Data type: `String`

Name of the temporary Cluster Information Base to use during the building
and commiting of the rule changes incorporated in this module.

##### `lvm_primitives`

Data type: `Array[String]`

Array containing the list of lvm primitives managed by the parent type. It
must contain an entry which matches the expected primitive described by the
volume_group parameter for this filesystem.

##### `volume_group`

Data type: `Sap::VgLvName`

Name of the LVM volume group to be administered. Used to generate the
device parameter.

##### `logical_volume`

Data type: `Sap::VgLvName`

Name of the logical volume containing this filesystem. This is used to
generate the device parameter.

##### `fstype`

Data type: `Sap::FsType`

Format of the filesystem.

##### `directory`

Data type: `Stdlib::AbsolutePath`

Mountpoint where this filesystem is attached.

##### `device`

Data type: `Stdlib::AbsolutePath`

Path to the underlying block device on which this filesystem resides.
Typically this is constructed with the volume group name and the logical
volume name.

Default value: "/dev/${volume_group}/${logical_volume}"

### sap::cluster::lvm

This defined type should be used by the sap::cluster class and not called
directly externally.

#### Examples

##### Definition for a shared commvault volume

```puppet
sap::cluster::as_inst { 'db2pr0_lvm_0':
  sid                => 'PR0',
  cib_name           => 'sap',
  volgrpname         => 'db2cv',
  exclusive          => true,
  partial_activation => false,
}
```

#### Parameters

The following parameters are available in the `sap::cluster::lvm` defined type.

##### `sid`

Data type: `Sap::SID`

System ID this instance is associated with

##### `cib_name`

Data type: `String`

Name of the temporary Cluster Information Base to use during the building
and commiting of the rule changes incorporated in this module.

##### `volgrpname`

Data type: `Sap::VgLvName`

Name of the LVM volume group to be administered.

##### `exclusive`

Data type: `Boolean`

When enabled the specified volume group will be exclusively activated.
see `pcs resource describe LVM` for details.

Default value: `true`

##### `partial_activation`

Data type: `Boolean`

When enabled corosync/pcs will attempt to activate the volume even if some
disks are missing. This can be extremely useful when LVM mirroring is
configured.

Default value: `false`

### sap::cluster::scsers

This defined type should be used by the sap::cluster class and not called
directly externally.

#### Examples

##### Asymmetric 4-node cluster with two app relevant nodes

```puppet
sap::cluster::scsers { 'PR0_ScsErs':
  sid       => 'PR0',
  cib_name  => 'sap',
  nodes     => {
    'pr0-app00.example.org' => ['app-scs'],
    'pr0-app01.example.org' => ['app-scs'],
    'pr0-dba.example.org'   => ['db-standard'],
    'pr0-dbb.example.org'   => ['db-standard'],
  },
  ers_ip    => '10.0.0.11'
  ers_inst  => {
    'type'   => 'ERS',
    'number' => '02',
    'host'   => 'pr0-ers',
  },
  scs_ip    => '10.0.0.10'
  scs_inst  => {
    'type'   => 'ASCS',
    'number' => '01',
    'host'   => 'pr0-cs',
  },
}
```

#### Parameters

The following parameters are available in the `sap::cluster::scsers` defined type.

##### `sid`

Data type: `Sap::SID`

System ID this instance is associated with

##### `cib_name`

Data type: `String`

Name of the temporary Cluster Information Base to use during the building
and commiting of the rule changes incorporated in this module.

##### `nodes`

Data type: `Sap::ClusterNodeRoles`

Mapping containing each a list of relevant roles for each node in the
cluster.

##### `ers_ip`

Data type: `Stdlib::IP::Address::Nosubnet`

IP address assigned to the ERS virtual host.

##### `ers_inst`

Data type: `Sap::InstData`

Structure providing details for the ERS instance. There are up to four
components. Note that the first three are mandatory, however, the fourth is
optional.
* `type` - Type of ERS instance. This should pretty much always be 'ERS'
* `number` - Two digit instance ID
* `host` - short-name of the ERS virtual host.
* `legacy_start_profile` - Boolean value indicating whether this is a sap
   instance with 'START_' profiles separate from the instance ala-pre NW 7.10

##### `scs_ip`

Data type: `Stdlib::IP::Address::Nosubnet`

IP address assigned to the SCS virtual host.

##### `scs_inst`

Data type: `Sap::InstData`

Structure providing details for the SCS instance. There are up to four
components. Note that the first three are mandatory, however, the fourth is
optional.
* `type` - Type of SCS instance. Could be either SCS or ASCS
* `number` - Two digit instance ID
* `host` - short-name of the SCS virtual host.
* `legacy_start_profile` - Boolean value indicating whether this is a sap
   instance with 'START_' profiles separate from the instance ala-pre NW 7.10

### sap::config::mount_point

This type is an implementation detail and should not be used directly!

#### Parameters

The following parameters are available in the `sap::config::mount_point` defined type.

##### `mount_path`

Data type: `String`

The path to the mountpoint. Note that this is a namevar.

Default value: $name

##### `file_params`

Data type: `Hash[String, String]`

Any valid parameter which could be provided to a File resource type can be
included in this hash. Note that ensure is always set to `directory`.

##### `count`

Data type: `Optional[Integer]`

Indicates that this path should be created $count times. The path must
contain the string `_N_` which will be substituted with the count.
For example `/db2/ECP/sapdata_N_`

Default value: `undef`

##### `mount_params`

Data type: `Optional[Hash]`

TODO - fill out this section

Default value: {}

##### `required_files`

Data type: `Optional[Array[String]]`

Ordered list of files which this mount point should require.

Default value: []

##### `mount_defaults`

Data type: `Optional[Hash]`

Hash of hashes containing mount-type specific defaults. Currently this is
used exclusively to provide default parameters to the NFSv4 client mounts
for SAP related connections.

Default value: {}

##### `per_sid`

Data type: `Sap::MountPointSIDMode`

Controls the validation behavior for the module via three possible states
`true` - the path is SID specific and an instance can be created for each
`false` - only one occurance of the path can exist on a system
`user` - the entry has SID specific attributes but the path is unique

##### `pattern_map`

Data type: `Hash[String, String]`

Hash mapping potential `pattern_values` to the regular expression patterns
which they should replace. The value retrieved by the same key in
`pattern_values` will be substituted into the mount_path, owner, and group
of the target file wherever the regular expression matches. Note that the
patterns should be provided as strings, however, they will be converted to
regexp for use via `Regexp($value)`

##### `pattern_values`

Data type: `Hash[String, String]`

Hash mapping substitution targets with their corresponding values for this
mountpoint instance. Typically substitutions are SID specific and are
specified globally via the `$sap::system_ids['pattern_map'] parameter.

### sap::management::db2_query_schedule

Implementation detail - a query file & associate cron schedule

#### Parameters

The following parameters are available in the `sap::management::db2_query_schedule` defined type.

##### `sid`

Data type: `Sap::SID`

System ID for this query

##### `query_file_name`

Data type: `String[1]`

Basename of the file defining this query. Note that this file must be a bash
script which implements the function `execute_query`.

##### `cron_entry`

Data type: `Sap::ScheduleEntry`

Cron schedule entry defining the frequency at which this query will run

##### `query_directory`

Data type: `Stdlib::AbsolutePath`

Location in which this (and all other) queries will be installed. Note that
this is set by the sap::management::db2_queries class

##### `schema`

Data type: `Optional[Sap::Db2Schema]`

In some cases the schema name of a given system does not match the SID. This
is typically the case for redirected restores in QA environments. If this
field is specified it overrides the standard "SAP${sid}" schema name.

Default value: `undef`

##### `output_target`

Data type: `Optional[Stdlib::AbsolutePath]`

Specifies the target result file of the scheduled query. Typically this only
applies to queries which perform some form of export statement.

Default value: `undef`

##### `output_group`

Data type: `Optional[String[1]]`

Operating system group which will be set as the owning group of the
output_target file after it is generated.

Default value: `undef`

##### `monitor_alias`

Data type: `Optional[Stdlib::Host]`

IPv4 address or DNS name representing a service IP that may be present on
one or more nodes. In the case of an HA/DR cluster execution of this query
will be restricted to the node which currently has this address.

Default value: `undef`

##### `query_content`

Data type: `Sap::SourceURL`



## Functions

### multiregsubst

Type: Ruby 4.x API

Performs a series of substitutions on the same target string

#### `multiregsubst(String $target, Hash[Regexp, String] $substmap)`

Performs a series of substitutions on the same target string

Returns: `String` Returns target after all substitutions

##### `target`

Data type: `String`

The string to be substituted

##### `substmap`

Data type: `Hash[Regexp, String]`

Hash mapping regexp to target value

### sap::cluster::filter_nodes

Type: Puppet Language

Simple function to retrieve only the nodes with a desired role

#### `sap::cluster::filter_nodes(Sap::NodeRole $target_role, Sap::ClusterNodeRoles $node_role_hash)`

The sap::cluster::filter_nodes function.

Returns: `Array[Stdlib::Fqdn]` Array of node names which have the target_role role.

##### `node_role_hash`

Data type: `Sap::ClusterNodeRoles`

Mapping from node names to a list of roles which that node can perform.

##### `target_role`

Data type: `Sap::NodeRole`

Nodes with this role will be returned in the resulting list

### sap::sysctl_calculated_values

Type: Puppet Language

Converts the provided data hash into the corresponding value. This class
performs recursive calls to construct a single value when provided with a
compound series of values.

#### `sap::sysctl_calculated_values(Array[Hash] $data, Integer $index)`

Converts the provided data hash into the corresponding value. This class
performs recursive calls to construct a single value when provided with a
compound series of values.

Returns: `String` The value computed by the calculation process.

##### `data`

Data type: `Array[Hash]`

Array of value hashes. The function will first attempt to call itself for
the next entry in the array and then prepend it's own calculation to
assemble the resulting string.

##### `index`

Data type: `Integer`

Current position in the provided array. The function will call itself for
index + 1 unless the current index is the last element in the array.

### sap::sysctl_line

Type: Puppet Language

This function converts a given entry from a module specific hash provided to
sap::params::config_sysctl into a line of the form <directive> = <value>.
There are three supported line types for this component:
1. `value` - Simply sets the value equal to the entry name
2. `calculated` - Computes the value via the following formula `base * multiplier / divisor`
3. `compound` - Performs a recursive call converting multiple value/calculated sub entries into a single space separated value.

#### `sap::sysctl_line(String $param, Hash[
    String,
    Variant[
      String,
      Integer,
      Array[
        Variant[
          String,
          Hash[
            String,
            Variant[
              Integer,
              String,
            ]
          ]
        ]
      ]
    ]
  ] $data)`

This function converts a given entry from a module specific hash provided to
sap::params::config_sysctl into a line of the form <directive> = <value>.
There are three supported line types for this component:
1. `value` - Simply sets the value equal to the entry name
2. `calculated` - Computes the value via the following formula `base * multiplier / divisor`
3. `compound` - Performs a recursive call converting multiple value/calculated sub entries into a single space separated value.

Returns: `String` A string of the form <directive> = <value>. For example:
`kernel.sem = 1250 256000 100 8192`

##### `param`

Data type: `String`

The name of the kernel parameter this corresponds to. E.g. `kernel.sem` is
the parameter for Linux kernel Semaphore parameters.

##### `data`

Data type: `Hash[
    String,
    Variant[
      String,
      Integer,
      Array[
        Variant[
          String,
          Hash[
            String,
            Variant[
              Integer,
              String,
            ]
          ]
        ]
      ]
    ]
  ]`

Detailed structure containing the components needed to produce the value for
the given parameter.

